

;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;SYSTEM-TEST BATCH PROGRAM FOR THE AIM-68000
;
;REVISION: A
;DATE CREATED: 8/9/82
;DATE LAST UPDATED: 8/9/82    REV: A    CHANGE:
;                   8/17/82   REV: A    CHANGE: UPDATE EXTERNAL FILE REFERENCES
;                   8/20/82   REV: A    CHANGE: RESTRUCTURE TO TEST STAND-ALONE
;                                               FIRST.  ELIMINATE REDUNDANT 
;                                               TESTS.
;                   8/27/82   REV: A    CHANGE: ADD TESTS 10-S AND 3-T.
;                   8/31/82   REV: A    CHANGE: REMOVE TEST 10-S UNTIL PROBLEM
;                                               IS FOUND.
;
;
;
;
;
;
;
;
;
;
;--------------------------------------------------------------
;TEST: 0-S
; This section initializes the system, and clears all previous settings.
;To begin, insure ONLY SWITCH #3 on the PERSONALITY MODULE is ON.
;POWER to the TARGET should be turned off.
*
;
INITIALIZE
CLEAR ALL
BREAK EVENT 1
SET ALL SYSTEM
MAP FROM 00000000 TO 0FFFFFFFF AS TARGET
MAP FROM 00000000 TO 0FFFFFFFF AS UNGUARDED
;
;--------------------------------------------------------------
;TEST: 1-S
; This test insures the MEMORY-MAPPING circuitry is working
*properly.
;
MAP FROM 00028000 TO 0002FFFF AS SYSTEM
;
; The above should indicate that 8 blocks of memory (from
;00028000 to 0002FFFF) are mapped with "PRIVILEGE = ALL"
;and "ACCESS = ALL".
;If this worked correctly, continue; else hit "Q" to exit.
*
;
;--------------------------------------------------------------
;TEST: 2-S
; The next test performs a walking-ones RAM test on the
;EMULATION RAM locations 00028000 to 00028030.
;
RAMTEST FROM 00028000 TO 00028030
;
;If this worked correctly, continue; else hit "Q" to exit.
*
;
;--------------------------------------------------------------
;TEST: 3-S
; This test checks the HOST-DOWNLOAD capability.
;
GET '[BALANCHUK.AIM]RAMTSTS.HEX'
;
;If this worked correctly, continue; else hit "Q" to exit.
*
;
;--------------------------------------------------------------
;TEST: 4-S
; This checks to insure the remaining untested EMULATION RAM
;works properly by executing a walking-ones RAM test from 
;the previously tested RAM area.  It also tests the SOFTWARE
;BREAKPOINT capability.
;
BREAK AT 00028028
BREAK AT 0002802E
EXECUTE 00028000
;
;The register display above should show:
;		'SOFTWARE BREAKPOINT'
;		D0:L = 00000001
;		A1:L = 00030000
;		PC:L = 0002802E
;If this worked correctly, continue; else hit "Q" to exit.
*
;
;--------------------------------------------------------------
;TEST: 5-S
; This test checks the register breakpoint feature as well as
;the TRACE function.
;
BREAK A1:L 00028032
GO 00028000
;
;The display above should show:
;		'REGISTER BREAKPOINT'
;		A1:L = 00028032
;		D0:L = 00000001
;		PC:L = 00028026
;If this worked correctly, continue; else hit "Q" to exit.
*
;
;--------------------------------------------------------------
;TEST: 6-S
; This section will show a REVIEW of the registers through the
*last 10 instructions executed.
;
REVIEW
;
;The last register display above should show:
;		A1:L = 00028032
;		D0:L = 00000001
;		PC:L = 00028026
;If this worked correctly, continue; else hit "Q" to exit.
*
;
;--------------------------------------------------------------
;TEST: 7-S
; This test checks the HARDWARE-BREAKPOINT capabilities.
;
CLEAR REGISTER
BREAK ADDRESS 00028012 00000000
BREAK TRIGGER ALL
EXECUTE 00028000
;
;The register display above should show:
;		'HARDWARE BREAKPOINT'
;		PC:L = 00028016
;		D0:L = 00000001
;		A1:L = 00028030
;If this worked correctly, continue; else hit "Q" to exit.
*
;
;--------------------------------------------------------------
;TEST: 8-S
; This test checks the TIMER BREAKPOINT feature.
;
CLEAR HARDWARE
BREAK AT 27 MSEC
EXECUTE 00028000
;
;The register display above should show:
;		'TIMER BREAKPOINT'
;		.039 ELAPSED TIME
;If this worked correctly, continue; else hit "Q" to exit.
*
;
;--------------------------------------------------------------
;TEST: 9-S
; This test checks the MEMORY PROTECTION circuitry.
;
CLEAR TIMER
MAP FROM 00029000 TO 00029FFF AS GUARDED
EXECUTE 00028000
;
;The register display above should show:
;		'TIMER BREAKPOINT'
;		PC:L = 00028018
;		A1:L = 00029000
;		D0:L = 00000001
;If this worked correctly, continue; else hit "Q" to exit.
*
;
;--------------------------------------------------------------
;
;
;
;
;The general logic works at full speed.
;The next test will test the target interface at low speed in stand-alone
;mode.
;
;
;
;
;
;-------------------------------------------------------------
;TURN ON TARGET POWER, and flip SWITCH #3 OFF.
*
;--------------------------------------------------------------
;
;The TARGET INTERFACE is now going to be tested.
;
;--------------------------------------------------------------
;TEST: 0-T
; This section initializes the system, and clears all previous settings.
;
INITIALIZE
SET ALL SYSTEM
MAP FROM 00029000 TO 00029FFF AS UNGUARDED
;
;--------------------------------------------------------------
;TEST: 1-T
; The following test checks the TARGET INTERFACE.
;
COPY FROM 00028000 TO 000280FF TO 00000100
CLEAR ALL
BREAK AT 00000128
BREAK AT 0000012E
EXECUTE 00000100
;
;The register display above should show:
;		'SOFTWARE BREAKPOINT'
;		PC:L = 0000012E
;		D0:L = 00000001
;		A1:L = 00030000
;If this worked correctly, continue; else hit "Q" to exit.
*
;--------------------------------------------------------------
;TEST: 2-T
; This test checks the HARDWARE-BREAKPOINT/EVENT-COUNT capabilities.
;
CLEAR REGISTER
REGISTER A1:L 00000130
FILL FROM 00000109 TO 0000010B WITH <00,40,00>
BREAK ADDRESS 00000116 00000000
BREAK TRIGGER ALL
BREAK EVENT 14
EXECUTE 0000010C
;
;The register display above should show:
;		'HARDWARE BREAKPOINT'
;		PC:L = 00000118
;		D0:L = 00000004
;		A1:L = 00000132
;If this worked correctly, continue; else hit "Q" to exit.
*
;
;--------------------------------------------------------------
;TEST: 3-T
; This tests the KEYBOARD ESCAPE feature.
;
*+++ WHEN THE PROGRAM BELOW BEGINS EXECUTION, QUICKLY HIT THE "ESC" KEY.
;
;
;
REGISTER A1:L 00000130
CLEAR HARDWARE
EXECUTE 0000010C
;
;The above register display should indicate:
;                     "KEYBOARD ESCAPE"
;If this is correct, continue; else hit "Q" to exit.
*
;-----------------------------------------------------------------------------
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;
;                 THE SYSTEM WORKS
;
;
;
;
;
;
;
;
;
;
;
;

